# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Brian Douglass bhdouglass@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gio, Gtk, Gdk, GObject, Notify, AppIndicator3 # pylint: disable=E0611
GObject.threads_init()

import logging
logger = logging.getLogger('indicator_remindor')

from indicator_remindor.PreferencesDialog import PreferencesDialog
from indicator_remindor.AboutDialog import AboutDialog
from indicator_remindor.ReminderDialog import ReminderDialog
from indicator_remindor.QuickDialog import QuickDialog
from indicator_remindor import indicator, helpers
from indicator_remindor.indicator_remindorconfig import get_data_file
from indicator_remindor.scheduler_gtk import SchedulerGtk

from remindor_common.helpers import ManageWindowInfo
from remindor_common.constants import *
from remindor_common import threads

import dbus
from dbus.mainloop.glib import DBusGMainLoop

import gettext
from gettext import gettext as _
gettext.textdomain('remindor-common')

class IndicatorRemindorWindow(Gtk.Window):
    __gtype_name__ = "IndicatorRemindorWindow"

    indicator_icons = ["indicator-remindor-active",
            get_data_file("media", "indicator-remindor-active.svg"), #so there is no file://
            get_data_file("media", "indicator-remindor-active_dark.svg"),
            "indicator-remindor"]

    setup_schedule = True

    today = None
    future = None
    past = None

    def __new__(cls):
        builder = helpers.get_builder('IndicatorRemindorWindow')
        new_object = builder.get_object("indicator_remindor_window")
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        logger.debug("Initializing Manage window")
        self.builder = builder
        self.ui = builder.get_ui(self, True)
        self.preferences_dialog = None
        self.dbus_service = None
        self.info = ManageWindowInfo(helpers.database_file())

        Notify.init("indicator-remindor")

        self.connect("delete-event", self.on_destroy)

        logger.debug("creating popup menu")
        self.popup = Gtk.Menu()
        add = Gtk.ImageMenuItem(Gtk.STOCK_EDIT)
        add.set_use_stock(True)
        add.set_label(_("Edit"))
        add.connect("activate", self.on_edit_clicked)
        add.show()
        self.popup.append(add)

        postpone_image = Gtk.Image()
        #postpone_image.set_from_stock(Gtk.STOCK_JUMP_TO, Gtk.IconSize.MENU)
        postpone_image.set_from_icon_name("go-jump", Gtk.IconSize.MENU)

        postpone = Gtk.ImageMenuItem()#Gtk.STOCK_JUMP_TO)
        #postpone.set_use_stock(True)
        postpone.set_use_stock(False)
        postpone.set_image(postpone_image)
        postpone.set_label(_("Postpone"))
        postpone.connect("activate", self.on_postpone_clicked)
        postpone.show()
        self.popup.append(postpone)

        delete = Gtk.ImageMenuItem(Gtk.STOCK_DELETE)
        delete.set_use_stock(True)
        delete.set_label(_("Delete"))
        delete.connect("activate", self.on_delete_clicked)
        delete.show()
        self.popup.append(delete)

        alarm_tree = self.builder.get_object("alarm_tree")
        alarm_tree.connect('button-release-event', self.tree_button_release)
        alarm_tree.connect('button-press-event', self.tree_click_release)
        logger.debug("done creating popup menu");

        logger.debug("setting up treeview cells");
        cell0 = Gtk.CellRendererText()
        cell0.set_property('background-set' , True)
        label_col = Gtk.TreeViewColumn(_("Label"), cell0, text=1, background=6)
        label_col.set_min_width(10)
        label_col.set_resizable(True)
        label_col.set_spacing(0)
        alarm_tree.append_column(label_col)

        cell2 = Gtk.CellRendererText()
        cell2.set_property('background-set' , True)
        time_col = Gtk.TreeViewColumn(_("Time"), cell2, text=2, background=6)
        time_col.set_min_width(10)
        time_col.set_resizable(True)
        time_col.set_spacing(0)
        alarm_tree.append_column(time_col)

        cell3 = Gtk.CellRendererText()
        cell3.set_property('background-set' , True)
        date_col = Gtk.TreeViewColumn(_("Date"), cell3, text=3, background=6)
        date_col.set_min_width(10)
        date_col.set_resizable(True)
        date_col.set_spacing(0)
        alarm_tree.append_column(date_col)

        cell1 = Gtk.CellRendererText()
        cell1.set_property('background-set' , True)
        notes_col = Gtk.TreeViewColumn(_("Notes"), cell1, text=4, background=6)
        notes_col.set_min_width(10)
        notes_col.set_resizable(True)
        notes_col.set_spacing(0)
        alarm_tree.append_column(notes_col)
        logger.debug("done setting up cells");

        self.indicator = indicator.new_application_indicator(self)
        self.indicator.get_menu().connect("show", self.clear_indicator)

        logger.debug("indicator --> icon: " + str(self.indicator.get_icon()) + ", theme: " + str(self.indicator.get_icon_theme_path()) + ", order: " + str(self.indicator.get_ordering_index()))

        istatus = "active"
        if self.indicator.get_status() == AppIndicator3.IndicatorStatus.PASSIVE:
            istatus = "passive"
        elif self.indicator.get_status() == AppIndicator3.IndicatorStatus.ATTENTION:
            istatus = "attention"
        logger.debug("indicator --> status: " + istatus)

        self.news = self.builder.get_object("news") #moved here for update_alarms()

        logger.debug("initializing scheduler and updating reminder list")
        self.scheduler = SchedulerGtk(self.indicator)
        self.scheduler.connect('update_schedule', self.update_alarm_schedule)
        self.update_alarms()

        GObject.timeout_add(1000 * 60 * 60 * 6, self.update_alarm_schedule) #update everything every 1/4 day

        b = threads.BlogReader(rssfeed, helpers.database_file())
        b.start()

        logger.debug("Manage window done initializing")

    def add_dbus_service(self, dbus_service):
        self.dbus_service = dbus_service
        self.scheduler.add_dbus_service(self.dbus_service)

    def on_help_clicked(self, widget, data=None):
        self.on_mnu_contents_activate(widget, data)

    def on_mnu_contents_activate(self, widget, data=None):
        helpers.show_uri(self, "ghelp:%s" % helpers.get_help_uri())

    def on_mnu_refresh_activate(self, widget, data=None):
        self.update_alarm_schedule()

    def on_mnu_clear_activate(self, widget, data=None):
        self.clear_indicator(None, True)

    def on_mnu_website_activate(self, widget, data=None):
        helpers.show_uri(self, website)

    def on_mnu_bugs_activate(self, widget, data=None):
        helpers.show_uri(self, bugsite)

    def on_mnu_question_activate(self, widget, data=None):
        helpers.show_uri(self, questionsite)

    def on_mnu_feature_activate(self, widget, data=None):
        helpers.show_uri(self, featuresite)

    def on_mnu_translate_activate(self, widget, data=None):
        helpers.show_uri(self, translatesite)

    def on_mnu_donate_activate(self, widget, data=None):
        helpers.show_uri(self, donatesite)

    def on_mnu_about_activate(self, widget, data=None):
        about = AboutDialog()
        about.set_transient_for(self)
        response = about.run()
        about.destroy()

    def on_news_clicked(self, widget, data=None):
        self.news.set_label(_("News"))
        helpers.show_uri(self, blogsite)

    def on_prefs_clicked(self, widget, data=None):
        self.on_mnu_preferences_activate(widget, data)

    def on_mnu_preferences_activate(self, widget, data=None):
        if self.preferences_dialog is not None:
            logger.debug('show existing preferences_dialog')
            self.preferences_dialog.present()
        else:
            logger.debug('create new preferences_dialog')
            self.preferences_dialog = PreferencesDialog()
            self.preferences_dialog.connect('destroy', self.on_preferences_dialog_destroyed)
            self.preferences_dialog.show()

    def on_mnu_close_activate(self, widget, data=None):
        self.hide_on_delete()
        return True

    def on_mnu_quit_activate(self, widget, data=None):
        self.on_indicator_quit_activate(widget, data=None)

    def on_indicator_quit_activate(self, widget, data=None):
        Gtk.main_quit()

    def on_destroy(self, widget, data=None):
        self.hide_on_delete()
        return True

    def on_indicator_manage_activate(self, widget, data=None):
        self.clear_indicator()
        self.update_alarms()
        self.show_all()

    def on_preferences_dialog_destroyed(self, widget, data=None):
        self.update_alarms()
        logger.debug('on_preferences_dialog_destroyed')
        self.preferences_dialog = None

    def on_mnu_quick_add_activate(self, widget, data=None):
        self.on_quick_add_clicked(widget, data)

    def on_quick_add_clicked(self, widget, data=None):
        logger.debug("on_quick_add_clicked")
        self.clear_indicator()

        quick_dialog = QuickDialog()
        quick_dialog.connect('alarm_added', self.add_to_schedule_slot)
        quick_dialog.run()
        quick_dialog.destroy()
        self.update_alarms()

    def on_mnu_add_activate(self, widget, data=None):
        self.on_add_clicked(widget, data)

    def on_add_clicked(self, widget, data=None):
        logger.debug("on_add_clicked")
        self.clear_indicator()

        reminder_dialog = ReminderDialog()
        reminder_dialog.connect('reminder_added', self.add_to_schedule_slot)

        response = reminder_dialog.run()
        while response == -11:
            response = reminder_dialog.run()

        self.update_alarms()

    def on_mnu_delete_activate(self, widget, data=None):
        self.delete_selected()

    def on_delete_clicked(self, widget, data=None):
        self.delete_selected()

    def on_mnu_edi_activate(self, widget, data=None):
        self.on_edit_clicked(widget, data)

    def on_edit_clicked(self, widget, data=None):
        logger.debug("on_edit_clicked")
        id, iter = self.get_selected_alarm()
        if id is not None and not id == "":
            reminder_dialog = ReminderDialog()
            reminder_dialog.connect('reminder_added', self.add_to_schedule_slot)

            model = self.builder.get_object("alarm_tree").get_model()
            id = model.get_value(iter, 0)

            reminder_dialog.edit(id)
            response = reminder_dialog.run()
            while response == -11:
                response = reminder_dialog.run()

            #done in ReminderDialog now
            #if response == Gtk.ResponseType.OK:
                #remove old alarm
                #self.delete_selected()

    def on_mnu_postpone_activate(self, widget, data=None):
        self.on_postpone_clicked(widget, data)

    def on_postpone_clicked(self, widget, data=None):
        logger.debug("on_postpone_clicked")
        alarm_id, iter = self.get_selected_alarm()
        if alarm_id is not None:
            if self.info.postpone(alarm_id):
                message = _("Sorry, you cannot postpone a repeating time.")
                dialog = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.INFO,
                    Gtk.ButtonsType.OK, message)
                dialog.run()
                dialog.destroy()

        self.update_alarms()

    def tree_button_release(self, widget, event, data=None):
        logger.debug("tree_button_release")
        id, row = self.get_selected_alarm()

        if event.button == 3 and id != "":
            x = int(event.x)
            y = int(event.y)
            etime = event.time

            alarm_tree = self.builder.get_object("alarm_tree")
            pthinfo = alarm_tree.get_path_at_pos(x, y)
            if pthinfo is not None:
                path, col, cellx, celly = pthinfo
                alarm_tree.grab_focus()
                alarm_tree.set_cursor(path, col, 0)
                self.popup.popup(None, None, None, None, event.button, etime)

    def tree_click_release(self, widget, event, data=None):
        logger.debug("tree_click_release")
        id, row = self.get_selected_alarm()
        if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS and id != "":
            self.on_edit_clicked(None)

    def delete_selected(self):
        logger.debug("delete_selected")
        alarm_id, iter = self.get_selected_alarm()
        if alarm_id is not None:
            self.info.delete(alarm_id)
            self.update_alarms()

    def get_selected_alarm(self):
        logger.debug("get_selected_alarm")
        alarm_tree = self.builder.get_object("alarm_tree")
        model, row = alarm_tree.get_selection().get_selected()

        alarm_id = None
        index = None
        if row is not None:
            if model.iter_is_valid(row):
                alarm_id = model.get_value(row, 0)

        return alarm_id, row

    def dbus_receiver(self, command):
        logger.debug("received " + command + " signal from dbus")

        if command == "update":
            self.update_alarm_schedule()
        elif command == "stop":
            logger.debug("dbus: stopping sound...")
            self.stop_sound()
        elif command == "manage":
            self.show()
        elif command == "close":
            Gtk.main_quit()
        elif command == "attention" or command == "active":
            pass #don't do anything, we probably sent this signal
        else:
            logger.debug("unrecognized dbus command: " + command)

    def update_alarm_schedule(self, widget=None, data=None):
        self.setup_schedule = True

        self.scheduler.clear_schedule()

        logger.debug("updating the whole schedule")
        self.update_alarms()
        return True

    def update_alarms(self):
        logger.debug("update_alarms")
        if self.setup_schedule:
            self.info.update(self.scheduler)
        else:
            self.info.update(None)

        alarm_tree_store = self.builder.get_object("alarm_tree_store")
        alarm_tree_store.clear()

        if self.info.show_news and self.info.new_news == 1:
            self.news.set_label(_("New News"))
        else:
            self.news.set_label(_("News"))

        if self.info.hide_indicator:
            if self.indicator.get_status() != AppIndicator3.IndicatorStatus.PASSIVE:
                self.indicator.set_status(AppIndicator3.IndicatorStatus.PASSIVE)
                logger.debug("setting indicator status to passive: " + str(self.indicator.get_status() == AppIndicator3.IndicatorStatus.PASSIVE))
        else:
            if self.indicator.get_status() != AppIndicator3.IndicatorStatus.ACTIVE:
                self.indicator.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
                logger.debug("setting indicator status to active: " + str(self.indicator.get_status() == AppIndicator3.IndicatorStatus.ACTIVE))

        iicon = self.indicator.get_icon()
        if iicon != self.indicator_icons[self.info.indicator_icon]:
            self.indicator.set_icon(self.indicator_icons[self.info.indicator_icon])

        logger.debug("update_alarms: setting up headers")
        today_s = _("Today's Reminders")
        future_s = _("Future Reminders")
        past_s = _("Past Reminders")

        self.today = alarm_tree_store.append(None, ["", today_s, "", "", "", today_s, self.info.today_color_hex])
        self.future = alarm_tree_store.append(None, ["", future_s, "", "", "", future_s, self.info.future_color_hex])
        self.past = alarm_tree_store.append(None, ["", past_s, "", "", "", past_s, self.info.past_color_hex])

        for a in self.info.reminder_list:
            logger.debug("update_alarms: adding alarm to list")
            logger.debug("(raw) label: " + a.label + " time: " + a.time + " date: " + a.date)

            parent = self.today
            if a.parent == self.info.future:
                parent = self.future
            elif a.parent == self.info.past:
                parent = self.past

            alarm_tree_store.append(parent, [str(a.id), a.label, a.time, a.date, a.note, a.tooltip, "#FFFFFF"])
            logger.debug("(fixed) label: " + a.label + " time: " + a.time + " date: " + a.date)

        alarm_tree = self.builder.get_object("alarm_tree")
        alarm_tree.expand_all()

        logger.debug("update_alarms: done setting up tree")

        if self.setup_schedule:
            self.setup_schedule = False

        return True

    def clear_indicator(self, widget=None, data=None):
        logger.debug("clear_indicator")
        logger.debug("widget: " + str(widget))
        logger.debug("data: " + str(data))
        if self.indicator.get_status() == AppIndicator3.IndicatorStatus.ATTENTION or data == True:
            self.indicator.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
        logger.debug("clear_indicator done")

        if self.dbus_service != None:
            logger.debug("emmiting dbus active signal")
            self.dbus_service.emitActive()

    def clear_indicator_dbus(self, entry_id, geometry):
        logger.debug("clear_indicator_dbus")
        if geometry[0] > 0 and self.indicator.get_status() == AppIndicator3.IndicatorStatus.ATTENTION:
            DBusGMainLoop(set_as_default=True)
            bus = dbus.SessionBus()
            proxy = bus.get_object('com.canonical.Unity.Panel.Service', '/com/canonical/Unity/Panel/Service')
            sync = proxy.Sync(dbus_interface='com.canonical.Unity.Panel.Service')

            for row in sync:
                if row[1] == entry_id and row[2] == self.indicator.get_id():
                    self.clear_indicator()
                    break

    def stop_sound(self, widget=None, data=None):
        logger.debug("stopping sound")
        self.scheduler.stop_sound()

        self.clear_indicator()

    def add_to_schedule_slot(self, widget, data=None):
        logger.debug("add_to_schedule_slot")
        self.scheduler.add_reminder_slot(widget, data)
        self.update_alarms()