# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Brian Douglass bhdouglass@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk, GObject # pylint: disable=E0611

import gettext
from gettext import gettext as _
gettext.textdomain('remindor-common')

from indicator_remindor import helpers
from remindor_common.helpers import QuickDialogInfo

class QuickDialog(Gtk.Dialog):
    __gtype_name__ = "QuickDialog"
    __gsignals__ = {
        'alarm_added' : (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,
            (GObject.TYPE_STRING,))
    }

    def __new__(cls):
        builder = helpers.get_builder('QuickDialog')
        new_object = builder.get_object('quick_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        self.builder = builder
        self.ui = builder.get_ui(self)

        self.info = QuickDialogInfo(helpers.database_file())

        self.label_object = self.builder.get_object("label_entry")
        self.label_object.connect('icon-release', self.label_clear)

        self.in_l = self.builder.get_object("in_label")
        #self.minutes_l = self.builder.get_object("minutes_label")
        self.minutes_object = self.builder.get_object("minutes_entry")
        self.time_combo = self.builder.get_object("time_combo")
        self.time_combo.set_active(self.info.unit)

        self.in_l2 = self.builder.get_object("in_label2")
        #self.minutes_l2 = self.builder.get_object("minutes_label2")
        self.minutes_object2 = self.builder.get_object("minutes_entry2")
        self.time_combo2 = self.builder.get_object("time_combo2")
        self.time_combo2.set_active(self.info.unit)

        self.message_object = self.builder.get_object("message_label")

        if self.info.show_slider:
            self.in_l.hide()
            #self.minutes_l.hide()
            self.minutes_object.hide()
            self.time_combo.hide()
        else:
            self.in_l2.hide()
            #self.minutes_l2.hide()
            self.minutes_object2.hide()
            self.time_combo2.hide()

        self.label_object.set_text(self.info.label)
        self.minutes_object.set_value(self.info.minutes)
        self.minutes_object2.set_value(self.info.minutes)
        self.message_object.set_text(self.info.info)

        if not self.info.show_info:
            self.message_object.hide()

    def label_clear(self, widget, pos, event, data=None):
        self.label_object.set_text("")

    def on_btn_add_clicked(self, widget, data=None):
        label = self.label_object.get_text()
        minutes = self.minutes_object.get_value()
        index = self.time_combo.get_active()

        if self.info.show_slider:
            minutes = self.minutes_object2.get_value()
            index = self.time_combo2.get_active()

        self.emit('alarm_added', self.info.reminder(label, minutes, index))

    def on_btn_cancel_clicked(self, widget, data=None):
        pass