# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Brian Douglass bhdouglass@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import optparse
import sys
import os

import gettext
from gettext import gettext as _
gettext.textdomain('remindor-common')

from gi.repository import Gtk # pylint: disable=E0611
import dbus
from dbus.mainloop.glib import DBusGMainLoop

#moved below
#from indicator_remindor import IndicatorRemindorWindow
from indicator_remindor.ReminderDialog import ReminderDialog
from indicator_remindor.QuickDialog import QuickDialog
from indicator_remindor.PreferencesDialog import PreferencesDialog
from indicator_remindor.helpers import check_autostart, check_database, log_file, config_dir
from indicator_remindor.indicator_remindorconfig import get_version

from remindor_common.helpers import parse_options, set_up_logging
from remindor_common.dbus_service import dbus_service

def main():
    #check database first so directory is already set up for parse_options()
    check_database()
    check_autostart()

    (options, parser) = parse_options(get_version())
    set_up_logging("indicator_remindor", log_file(), config_dir(), options)

    DBusGMainLoop(set_as_default=True)
    session_bus = dbus.SessionBus()
    #name = dbus.service.BusName('tk.bhdouglass.indicator_remindor', session_bus)
    ds = dbus_service(session_bus)

    if options.add:
        dialog = ReminderDialog()
        response = dialog.run()
        while response == -11:
            response = dialog.run()

        dialog.destroy()
        ds.emitUpdate()
        sys.exit(0)

    elif options.quick:
        dialog = QuickDialog()
        dialog.run()
        dialog.destroy()
        ds.emitUpdate()
        sys.exit(0)

    elif options.manage:
        ds.emitManage()
        sys.exit(0)

    elif options.prefs:
        dialog = PreferencesDialog()
        response = dialog.run()
        while response == -11:
            response = dialog.run()

        dialog.destroy()
        ds.emitUpdate()
        sys.exit(0)

    elif options.stop:
        ds.emitStop()
        sys.exit(0)

    elif options.update:
        ds.emitUpdate()
        sys.exit(0)

    else:
        from indicator_remindor import IndicatorRemindorWindow
        #for delaying load of gst --> https://bugzilla.gnome.org/show_bug.cgi?id=425847

        window = IndicatorRemindorWindow.IndicatorRemindorWindow()
        window.add_dbus_service(ds)
        #window.show()

        bus = dbus.SystemBus()
        bus.add_signal_receiver(window.update_alarm_schedule, signal_name='Resuming',
            dbus_interface='org.freedesktop.UPower', path='/org/freedesktop/UPower')

        bus2 = dbus.SessionBus()
        bus2.add_signal_receiver(window.clear_indicator_dbus, signal_name='EntryActivated',
            dbus_interface='com.canonical.Unity.Panel.Service', path='/com/canonical/Unity/Panel/Service')

        bus2.add_signal_receiver(window.dbus_receiver, signal_name=None,
            dbus_interface=ds.interface(), path=ds.path())

    Gtk.main()