# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Brian Douglass bhdouglass@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk, GObject, Gdk # pylint: disable=E0611

from indicator_remindor import helpers
from remindor_common.datetimeutil import simplify_singular_time, simplify_singular_date
from remindor_common.helpers import DateDialogInfo, TimeDialogInfo

import gettext
from gettext import gettext as _
gettext.textdomain('remindor-common')

class DateDialog(Gtk.Dialog):
    __gtype_name__ = "DateDialog"
    __gsignals__ = {
        'date_updated' : (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,
            (GObject.TYPE_STRING,))
    }

    date_format = 0

    def __new__(cls):
        builder = helpers.get_builder('DateDialog')
        new_object = builder.get_object('date_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        self.builder = builder
        self.ui = builder.get_ui(self)

        self.info = None

        #labels
        self.date_on_l = self.builder.get_object("date_on_label")
        self.date_every_l = self.builder.get_object("date_every_label")
        self.date_days_l = self.builder.get_object("date_days_label")
        self.date_every2_l = self.builder.get_object("date_every_label2")
        self.date_from_l = self.builder.get_object("date_from_label")
        self.date_to_l = self.builder.get_object("date_to_label")
        self.error_l = self.builder.get_object("error_label")
        self.error_l.hide()

        #boxes
        self.date_every_b = self.builder.get_object("date_every_box")

        #entries/combos
        self.date_c = self.builder.get_object("date_combo")
        self.date_once_c = self.builder.get_object("date_once_combo")
        self.date_once_e = self.builder.get_object("date_once_entry")
        self.date_every_e = self.builder.get_object("date_every_entry")
        self.date_every_c = self.builder.get_object("date_every_combo")
        self.date_from_e = self.builder.get_object("date_from_entry")
        self.date_to_e = self.builder.get_object("date_to_entry")

        self.ok = self.builder.get_object("btn_ok")

        #adjustments
        self.days_a = self.builder.get_object("days_adjustment")
        self.month_a = self.builder.get_object("month_adjustment")

        #connections
        self.date_c.connect('changed', self.date_changed)
        self.date_once_c.connect('changed', self.date_once_changed)
        self.date_once_e.connect('changed', self.validate_date)
        self.date_once_e.connect('icon-release', self.clear_widget)
        self.date_from_e.connect('changed', self.validate_date)
        self.date_from_e.connect('icon-release', self.clear_widget)
        self.date_to_e.connect('changed', self.validate_date)
        self.date_to_e.connect('icon-release', self.clear_widget)

        self.date_c.set_active(0)
        self.date_once_c.set_active(0)
        self.date_every_c.set_active(0)

    def on_delete_event(self, widget, data=None):
        self.destroy()

    def on_btn_ok_clicked(self, widget, data=None):
        index = self.date_c.get_active()
        once = self.date_once_c.get_active()
        once_date = self.date_once_e.get_text()
        every = self.date_every_c.get_active()
        every_spin = self.date_every_e.get_value()
        from_date = self.date_from_e.get_text()
        to_date = self.date_to_e.get_text()

        value = self.info.build_date(index, once, once_date, every, every_spin, from_date, to_date)

        self.emit('date_updated', value)
        self.destroy()

    def validate_date(self, widget, data=None):
        once_index = self.date_once_c.get_active()
        once_s = self.date_once_e.get_text()
        from_s = self.date_from_e.get_text()
        to_s = self.date_to_e.get_text()
        index = self.date_c.get_active()
        sensitive = True

        if self.info != None:
            status = self.info.validate_date(index, once_index, once_s, from_s, to_s)

            if status == self.info.once_error:
                self.date_once_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
                sensitive = False
            else:
                self.date_once_e.set_icon_from_stock(0, None)

            if status == self.info.from_error or status == self.info.from_to_error:
                self.date_from_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
                sensitive = False
            else:
                self.date_from_e.set_icon_from_stock(0, None)

            if status == self.info.to_error or status == self.info.from_to_error:
                self.date_to_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
                sensitive = False
            else:
                self.date_to_e.set_icon_from_stock(0, None)

            if status == self.info.order_error:
                self.error_l.show()
                sensitive = False
            else:
                self.error_l.hide()

        self.ok.set_sensitive(sensitive)

    def on_btn_cancel_clicked(self, widget, data=None):
        self.destroy()

    def clear_widget(self, widget, pos, event, data=None):
        widget.set_text("")

    def set_date_format(self, df):
        self.date_format = df

    def set_data(self, data): #should be formated with datetimeutil.str_date_simplify() and datetimeutil.fix_date_format()
        self.info = DateDialogInfo(data, helpers.database_file())

        self.date_c.set_active(self.info.active)
        self.date_once_c.set_active(self.info.once_active)
        self.date_once_e.set_text(self.info.once_date)
        self.date_every_c.set_active(self.info.every_active)
        self.date_every_e.set_value(self.info.every_spin)
        self.date_from_e.set_text(self.info.from_date)
        self.date_to_e.set_text(self.info.to_date)

    def date_once_changed(self, widget, data=None):
        index = widget.get_active()
        self.validate_date(None)

        if index == 0:
            self.date_once_e.show()
        else:
            self.date_once_e.hide()

    def date_changed(self, widget, data=None):
        self.hide_date()
        self.validate_date(None)

        index = widget.get_active()
        if index == 0: #once
            self.date_on_l.show()
            self.date_once_c.show()
            #self.date_once_e.show()
            self.date_once_changed(self.date_once_c)
        elif index == 1: #every day
            self.date_from_l.show()
            self.date_to_l.show()
            self.date_from_e.show()
            self.date_to_e.show()
        elif index == 2: #every X days
            self.date_every_l.show()
            self.date_every_l.set_text(_("Every"))
            self.date_every_b.show()
            self.date_every_e.set_adjustment(self.days_a)
            self.date_days_l.show()
            self.date_days_l.set_text(_("Day(s)"))
            self.date_from_l.show()
            self.date_to_l.show()
            self.date_from_e.show()
            self.date_to_e.show()
        elif index == 3: #every Xth of the month
            self.date_every_l.show()
            self.date_every_l.set_text(_("Every"))
            self.date_every_b.show()
            self.date_every_e.set_adjustment(self.month_a)
            self.date_every_e.set_value(1)
            self.date_days_l.show()
            self.date_days_l.set_text(_("of the month"))
            self.date_from_l.show()
            self.date_to_l.show()
            self.date_from_e.show()
            self.date_to_e.show()
        elif index == 4: #every <day>
            self.date_every2_l.show()
            self.date_every_c.show()
            self.date_from_l.show()
            self.date_to_l.show()
            self.date_from_e.show()
            self.date_to_e.show()
        elif index == 5: #every other day
            self.date_from_l.show()
            self.date_to_l.show()
            self.date_from_e.show()
            self.date_to_e.show()
        elif index == 6: #Next X days
            self.date_every_l.show()
            self.date_every_l.set_text(_("Next"))
            self.date_every_b.show()
            self.date_every_e.set_adjustment(self.days_a)
            self.date_days_l.show()
            self.date_days_l.set_text(_("Day(s)"))
            self.date_from_l.show()
            self.date_from_e.show()

    def hide_date(self):
        self.date_on_l.hide()
        self.date_every_l.hide()
        self.date_days_l.hide()
        self.date_every2_l.hide()
        self.date_from_l.hide()
        self.date_to_l.hide()
        self.date_once_c.hide()
        self.date_once_e.hide()
        self.date_every_b.hide()
        self.date_every_c.hide()
        self.date_from_e.hide()
        self.date_to_e.hide()

class TimeDialog(Gtk.Dialog):
    __gtype_name__ = "TimeDialog"
    __gsignals__ = {
        'time_updated' : (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,
            (GObject.TYPE_STRING,))
    }

    def __new__(cls):
        builder = helpers.get_builder('TimeDialog')
        new_object = builder.get_object('time_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        self.builder = builder
        self.ui = builder.get_ui(self)

        self.info = None

        #labels
        self.time_at_l = self.builder.get_object("time_at_label")
        self.time_every_l = self.builder.get_object("time_every_label")
        self.time_minutes_hours_l = self.builder.get_object("time_minute_hour_label")
        self.time_from_l = self.builder.get_object("time_from_label")
        self.time_to_l = self.builder.get_object("time_to_label")
        self.error_l = self.builder.get_object("error_label")
        self.error_l.hide()

        #boxes
        self.time_every_b = self.builder.get_object("time_every_box")

        #entries/combos
        self.time_c = self.builder.get_object("time_combo")
        self.time_once_e = self.builder.get_object("time_once_entry")
        self.time_every_e = self.builder.get_object("time_every_entry")
        self.time_from_e = self.builder.get_object("time_from_entry")
        self.time_to_e = self.builder.get_object("time_to_entry")

        self.ok = self.builder.get_object("btn_ok")

        #connections
        self.time_c.connect('changed', self.time_changed)
        self.time_once_e.connect('changed', self.validate_time)
        self.time_once_e.connect('icon-release', self.clear_widget)
        self.time_from_e.connect('changed', self.validate_time)
        self.time_from_e.connect('icon-release', self.clear_widget)
        self.time_to_e.connect('changed', self.validate_time)
        self.time_to_e.connect('icon-release', self.clear_widget)

        self.time_c.set_active(0)

    def on_btn_ok_clicked(self, widget, data=None):
        index = self.time_c.get_active()
        once_s = self.time_once_e.get_text()
        every = self.time_every_e.get_value()
        from_s = self.time_from_e.get_text()
        to_s = self.time_to_e.get_text()
        value = self.info.build_time(index, once_s, every, from_s, to_s)

        self.emit('time_updated', value)
        self.destroy()

    def on_btn_cancel_clicked(self, widget, data=None):
        self.destroy()

    def on_delete_event(self, widget, data=None):
        self.destroy()

    def validate_time(self, widget=None, data=None):
        once_s = self.time_once_e.get_text()
        from_s = self.time_from_e.get_text()
        to_s = self.time_to_e.get_text()
        index = self.time_c.get_active()
        sensitive = True

        if not self.info == None:
            error = self.info.validate_time(index, once_s, from_s, to_s)
            if error != self.info.no_error:
                sensitive = False

            if error == self.info.once_error:
                self.time_once_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
            else:
                self.time_once_e.set_icon_from_stock(0, None)

            if error == self.info.order_error:
                self.error_l.show()
            else:
                self.error_l.hide()

            if error == self.info.from_to_error:
                self.time_from_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
                self.time_to_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)

            if error == self.info.from_error:
                self.time_from_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
            else:
                self.time_from_e.set_icon_from_stock(0, None)

            if error == self.info.to_error:
                self.time_to_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
            else:
                self.time_to_e.set_icon_from_stock(0, None)

        '''if index == 0:
            if simplify_singular_time(once_s) == None:
                self.time_once_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
                sensitive = False
            else:
                self.time_once_e.set_icon_from_stock(0, None)
        else:
            if (to_s != "" and from_s == "") or (from_s != "" and simplify_singular_time(from_s) == None):
                self.time_from_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
                sensitive = False
            else:
                self.time_from_e.set_icon_from_stock(0, None)

            if (to_s == "" and from_s != "") or (to_s != "" and simplify_singular_time(to_s) == None):
                self.time_to_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
                sensitive = False
            else:
                self.time_to_e.set_icon_from_stock(0, None)'''

        self.ok.set_sensitive(sensitive)

    def clear_widget(self, widget, pos, event, data=None):
        widget.set_text("")

    def set_data(self, data): #should be formated with datetimeutil.str_time_simplify() and datetimeutil.fix_time_format()
        self.info = TimeDialogInfo(data, helpers.database_file())

        self.time_c.set_active(self.info.active)
        self.time_once_e.set_text(self.info.once_s)
        self.time_every_e.set_value(self.info.every)
        self.time_from_e.set_text(self.info.from_s)
        self.time_to_e.set_text(self.info.to_s)

    def time_changed(self, widget, data=None):
        self.hide_time()
        self.validate_time()

        index = widget.get_active()
        if index == 0: #once
            self.time_at_l.show()
            self.time_once_e.show()
        elif index == 1 or index == 2: #every X minutes/hours
            self.time_every_l.show()
            self.time_every_b.show()
            self.time_from_l.show()
            self.time_from_e.show()
            self.time_to_l.show()
            self.time_to_e.show()

            if index == 1: #minutes
                self.time_minutes_hours_l.set_text(_("Minute(s)"))
            else: #hours
                self.time_minutes_hours_l.set_text(_("Hour(s)"))

    def hide_time(self):
        self.time_at_l.hide()
        self.time_every_l.hide()
        self.time_from_l.hide()
        self.time_to_l.hide()
        self.time_every_b.hide()
        self.time_once_e.hide()
        self.time_from_e.hide()
        self.time_to_e.hide()

class CommandDialog(Gtk.Dialog):
    __gtype_name__ = "CommandDialog"
    __gsignals__ = {
        'command' : (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,
            (GObject.TYPE_STRING,))
    }

    manpage_url = "http://manpages.ubuntu.com/cgi-bin/search.py?q="

    insert_values = ["+date+", "+month+", "+monthname+", "+day+", "+dayname+", "+yearday+",
                          "+year+", "+time+", "+hour+", "+hour12+", "+minute+", "+second+",
                          "+microsecond+", "+soundfullfile+", "+soundfile+"]

    def __new__(cls):
        builder = helpers.get_builder('CommandDialog')
        new_object = builder.get_object('command_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        self.builder = builder
        self.ui = builder.get_ui(self)

        self.appchooser = self.builder.get_object("appchooser")
        self.command_e = self.builder.get_object("command_entry")
        self.insert_e = self.builder.get_object("insert_entry")
        self.insert_e.set_active(0)

        self.command_e.connect('icon-release', self.clear_entry)
        self.appchooser.connect('application-selected', self.select_app)

    def set_command(self, command):
        self.command_e.set_text(command)

    def clear_entry(self, widget, pos, event, data=None):
        widget.set_text("")

    def select_app(self, info=None, data=None):
        buffer = self.command_e.get_buffer()
        buffer.insert_text(0, self.get_app() + " ", -1)

    def on_insert_button_clicked(self, widget, data=None):
        active = self.insert_e.get_active()

        buffer = self.command_e.get_buffer()
        pos = self.command_e.get_position()
        buffer.insert_text(pos, self.insert_values[active], -1)

    def on_btn_help_clicked(self, widget, data=None):
        helpers.show_uri(self, self.manpage_url + self.get_app())

    def on_btn_select_clicked(self, widget, data=None):
        self.emit('command', self.command_e.get_text())
        self.destroy()

    def on_btn_cancel_clicked(self, widget, data=None):
        self.destroy()

    def on_delete_event(self, widget, data=None):
        self.destroy()

    def get_app(self):
        info = self.appchooser.get_app_info()
        return info.get_executable()