# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Brian Douglass bhdouglass@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from indicator_remindor.helpers import database_file
from remindor_common.scheduler import GenericScheduler
from gi.repository import Gtk, Gdk, GObject, Notify, AppIndicator3, Gst
Gst.init(None)

import gettext
from gettext import gettext as _
gettext.textdomain('remindor-common')

import logging
logger = logging.getLogger('indicator_remindor')

class SchedulerGtk(GenericScheduler, Gtk.Widget):
    __gsignals__ = {
        'update_schedule' : (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE, ())
    }

    def __init__(self, indicator):
        GenericScheduler.__init__(self, database_file())
        Gtk.Widget.__init__(self)
        self.indicator = indicator

        playbin = "playbin2"
        gst_version = Gst.version()
        if gst_version[0] > 0:
            playbin = "playbin" #gst version 1.0's playbin is gst version 0.10's playbin2

        self.player = Gst.ElementFactory.make(playbin, "player")

    def remove_reminder(self, reminder):
        logger.debug("schedulergtk: remove_reminder")
        GObject.source_remove(reminder)

    def change_icon(self):
        logger.debug("schedulergtk: change_icon")
        self.indicator.set_status(AppIndicator3.IndicatorStatus.ATTENTION)

        if self.dbus_service != None:
            logger.debug("emmiting dbus attention signal")
            self.dbus_service.emitAttention()

    def play_sound_helper(self, sound_file, sound_loop, sound_loop_times):
        self.player.set_state(Gst.State.READY)
        self.play_filename = "file://" + sound_file
        self.player.set_property("uri", self.play_filename)
        self.player.set_state(Gst.State.PLAYING)
        self.play_count = 0

        if sound_loop_times:
            self.player.connect("about-to-finish", self.on_about_to_finish)

    def stop_sound_helper(self):
        self.player.set_state(Gst.State.READY)

    def remove_playing_sound(self):
        logger.debug("schedulergtk: remove_playing_sound")
        GObject.source_remove(self.playing_sound)

    def add_playing_sound(self, length):
        logger.debug("schedulergtk: add_playing_sound")
        self.playing_sound = GObject.timeout_add(int(length * 1000), self.stop_sound, None)

    def popup_notification(self, label, notes):
        logger.debug("schedulergtk: popup_notification: " + label + " " + notes)

        n = Notify.Notification.new(label, notes, "indicator-remindor")
        n.show()

    def popup_dialog(self, label, notes):
        logger.debug("schedulergtk: popup_dialog: " + label + " " + notes)

        image = Gtk.Image()
        image.set_from_icon_name("indicator-remindor", Gtk.IconSize.DIALOG)
        image.show()

        dialog = Gtk.MessageDialog(flags=Gtk.DialogFlags.MODAL, message_format=notes)
        dialog.set_image(image)
        dialog.set_title(label)
        dialog.show()

    def update_schedule(self):
        logger.debug("schedulergtk: update_schedule")
        self.emit('update_schedule')

    def on_about_to_finish(self, player):
        logger.debug("scheduler: on_about_to_finish")
        if self.play_count < (self.sound_loop_times - 1):
            logger.debug("looping sound: " + str(self.play_count + 1) + "/" + str(self.sound_loop_times - 1))

            self.player.set_property("uri", self.play_filename)
            self.play_count += 1

    def add_to_schedule(self, delay, id):
        logger.debug("schedulergtk: add_to_schedule, delay: " + str(delay))
        if delay <= 172800: #two days
            self.schedule.append(GObject.timeout_add(int(delay * 1000), self.run_alarm, id))
        else:
            logger.debug("schedulergtk: not adding reminder with id=" + str(id) + " because it is more than 172800 seconds/2 days out");